// AVX (256bit register ymm0-ymm16 = 4*f32) + FMA (fused-multiply-add)
//   some CPUs? gcc8? does not support mix AVX + FMA automatically?
// gcc   -O2 -mavx2 -mfma            # since 4.9 -S  PC16.gcc821=OK
// gcc   -O2 -march=skylake ../utils/fma_avx.c # -S  PC16.gcc821=OK
// clang -O1 -mavx2 -mfma -ffp-contract=fast
//__m256		256-bit vector containing 8 floats	
//__m256d		256-bit vector containing 4 doubles
//__m256i		256-bit vector containing integers
#include <immintrin.h>
#include <stdio.h>

#pragma GCC optimize ("O2","tree-vectorize","fast-math")

__m256 float_mul_addv(__m256 a, __m256 b, __m256 c) {
    return _mm256_add_ps(_mm256_mul_ps(a, b), c);
}

__m256d double_mul_addv(__m256d a, __m256d b, __m256d c) {
    return _mm256_add_pd(_mm256_mul_pd(a, b), c);
}

// do not optimize function-call away
#pragma GCC optimize ("O0","tree-vectorize","fast-math")
int main(int argn, char *argv[]) {
  float x1 = 1.0 + ((argn>1) ? atof(argv[1]) : 0.0);
  float x2 = 2.0 + ((argn>2) ? atof(argv[2]) : 0.0);
  float x3 = 3.0 + ((argn>3) ? atof(argv[3]) : 0.0);
  __m256 a = _mm256_set_ps(2.0, 4.0, 6.0, 8.0, 10.0, x3, x2, x1);
  __m256 b = _mm256_set_ps(x1,  3.0,  x3, 7.0,  9.0, 11.0, 13.0, 15.0);
  __m256 c = _mm256_set_ps(1.0,  x2,  x3, 7.0,  9.0, 11.0, 13.0, 15.0);

  /* Compute the difference between the two vectors */
  // __m256 result = _mm256_sub_ps(a, b);
  __m256 r = float_mul_addv(a, b, c);
  
  float* f = (float*)&r;
  printf("%f %f %f %f %f %f %f %f\n",
    f[0], f[1], f[2], f[3], f[4], f[5], f[6], f[7]);

  return 0;
}